using Godot;
using System;
#if REAL_T_IS_DOUBLE
using real_t = System.Double;
#else
using real_t = System.Single;
#endif

/// <summary>
/// This node converts a 3D position to 2D using a 2.5D transformation matrix.
/// The transformation of its 2D child is therefore controlled by its own transform.
/// </summary>
public class Node25D : Node2D
{
    private Spatial spatialNode;
    private Transform25D transform25D;

    public override void _Ready()
    {
        Node25DReady();
    }

    public override void _Process(real_t delta)
    {
        Node25DProcess();
    }

    /// <summary>
    /// Call this method in _Ready, or before you run Node25DProcess.
    /// </summary>
    protected void Node25DReady()
    {
        spatialNode = GetChild<Spatial>(0);
        transform25D = new Transform25D(Basis25D.FortyFive * 32);
        // Changing FortyFive here will change the default for all Node25D instances.
        // 32 is the number of 2D units in one 3D unit.
        // The Godot logo is 64x64 and I made the box extents=1 so I used 32 here.
    }
    
    /// <summary>
    /// Call this method in _Process, or whenever the position of this object changes.
    /// </summary>
    protected void Node25DProcess()
    {
        if (Input.IsActionJustPressed("TopDownMode"))
        {
            transform25D.basis = Basis25D.TopDown * 32;
        }
        else if (Input.IsActionJustPressed("FrontSideMode"))
        {
            transform25D.basis = Basis25D.FrontSide * 32;
        }
        else if (Input.IsActionJustPressed("FortyFiveMode"))
        {
            transform25D.basis = Basis25D.FortyFive * 32;
        }
        else if (Input.IsActionJustPressed("IsometricMode"))
        {
            transform25D.basis = Basis25D.Isometric * 32;
        }
        else if (Input.IsActionJustPressed("ObliqueYMode"))
        {
            transform25D.basis = Basis25D.ObliqueY * 32;
        }
        else if (Input.IsActionJustPressed("ObliqueZMode"))
        {
            transform25D.basis = Basis25D.ObliqueZ * 32;
        }
        
        // The actual math.
        transform25D.spatialPosition = spatialNode.Transform.origin;
        GlobalTransform = transform25D.FlatTransform;
    }
    
}
