using Godot;
using System;
#if REAL_T_IS_DOUBLE
using real_t = System.Double;
#else
using real_t = System.Single;
#endif

/// <summary>
/// Calculates the 2D transformation from a 3D position and a 25D basis.
/// </summary>
public struct Transform25D
{
    // Public fields store information that is used to calculate the properties.
    
    /// <summary>
    /// Controls how the 3D position is transformed into 2D.
    /// </summary>
    public Basis25D basis;
    
    /// <summary>
    /// The 3D position of the object. Should be updated on every frame before everything else.
    /// </summary>
    public Vector3 spatialPosition;

    // Public properties calculate on-the-fly.
    
    /// <summary>
    /// The full 2D transformation, including scale.
    /// TODO: Scale doesn't work.
    /// </summary>
    public Transform2D FlatTransform {
        get
        {
            return new Transform2D(0, FlatPosition);
        }
    }
    
    /// <summary>
    /// The 2D position of this object. Faster than FlatTransform, but no scale.
    /// </summary>
    public Vector2 FlatPosition {
        get
        {
            Vector2 pos = spatialPosition.x * basis.x;
                   pos += spatialPosition.y * basis.y;
                   pos += spatialPosition.z * basis.z;
            return pos;
        }
    }

    public Transform25D(Basis25D basis25D)
    {
        basis = basis25D;
        spatialPosition = Vector3.Zero;
    }

    public Transform25D(Basis25D basis25D, Vector3 position3D)
    {
        basis = basis25D;
        spatialPosition = position3D;
    }
}
