using Godot;
using System;
#if REAL_T_IS_DOUBLE
using real_t = System.Double;
#else
using real_t = System.Single;
#endif

/// <summary>
/// Handles Player-specific behavior like moving. We calculate such things with KinematicBody.
/// </summary>
public class Player25D : KinematicBody
{
    public override void _Ready()
    {
        //SetAxisLock();
        // TODO: How the heck does this work?
        // It's not vital but I'd like to lock rotation, since we don't need that.
    }

    public override void _Process(real_t delta)
    {
        HorizontalMovement(delta);
        VerticalMovement(delta);
        if (Input.IsActionPressed("ResetPosition"))
        {
            Transform = new Transform(Basis.Identity, Vector3.Up * 10);
            verticalSpeed = 0;
        }
    }
    
    /// <summary>
    /// Checks WASD and Shift for horizontal movement via MoveAndSlide.
    /// </summary>
    /// <param name="delta">Time delta since last call</param>
    private void HorizontalMovement(real_t delta)
    {
        Vector3 localX = Vector3.Right;
        Vector3 localZ = Vector3.Back;

        // Gather player input and add directional movement to force Vector3 variables
        Vector3 moveDir = Vector3.Zero;
        if (Input.IsActionPressed( "MoveRight"  )) { moveDir += localX; }
        if (Input.IsActionPressed(  "MoveLeft"  )) { moveDir -= localX; }
        if (Input.IsActionPressed("MoveForward" )) { moveDir -= localZ; }
        if (Input.IsActionPressed("MoveBackward")) { moveDir += localZ; }
        
        moveDir = moveDir.Normalized() * delta * 300;
        if (Input.IsActionPressed("MovementBoost"))
        {
            moveDir *= 5;
        }
        MoveAndSlide(moveDir);
    }
    
    private real_t verticalSpeed = 0;
    /// <summary>
    /// Checks Jump and applies gravity and vertical speed via MoveAndCollide.
    /// </summary>
    /// <param name="delta">Time delta since last call</param>
    private void VerticalMovement(real_t delta)
    {
        if (Input.IsActionJustPressed("Jump"))
        {
            verticalSpeed = 1.25f;
        }
        verticalSpeed -= delta * 5; // Gravity
        var k = MoveAndCollide(Vector3.Up * verticalSpeed);
        if (k != null) {
            verticalSpeed = 0;
        }
    }
}
